@@ -13,7 +13,6 @@ import traceback
 import subprocess as sp
 
 from os.path import dirname, join
-from os import environ, listdir
 
 from doge import wow
 
@@ -26,9 +25,7 @@ class Doge(object):
     def __init__(self, tty, doge_path=default_doge):
         self.tty = tty
         self.doge_path = doge_path
-
-        self.real_data = []
-        self.extra_words = []
+        self.words = wow.WORDS
 
         # If owna wants his doge instead, let that be. All doges are wow.
         self._no_override_doge = False
@@ -56,7 +53,7 @@ class Doge(object):
 
         # Check for prompt height so that we can fill the screen minus how high
         # the prompt will be when done.
-        prompt = environ.get('PS1', '').split('\n')
+        prompt = os.environ.get('PS1', '').split('\n')
         line_count = len(prompt) + 1
 
         # Create a list filled with empty lines and Shibe at the bottom.
@@ -64,12 +61,12 @@ class Doge(object):
         self.lines = ['\n' for x in fill]
         self.lines += doge
 
-        # Get some system data
-        # TODO: Refactor to be stateless
-        self.get_real_data()
-
         # Try to fetch data fed thru stdin
-        self.get_stdin_data()
+        had_stdin = self.get_stdin_data()
+
+        # Get some system data, but only if there was nothing in stdin
+        if not had_stdin:
+            self.get_real_data()
 
         # Apply the text around Shibe
         self.apply_text()
@@ -108,7 +105,7 @@ class Doge(object):
                         pass
 
                 # Same for text.
-                self.extra_words.extend(list(holiday_setup.get('words', [])))
+                self.words.extend(list(holiday_setup.get('words', [])))
                 break
 
     def apply_text(self):
@@ -117,43 +114,23 @@ class Doge(object):
 
         """
 
-        # TODO: Refactor the shuffling so that it is not true random but rather
-        # a shuffled start that is iterated over. This should remove at least
-        # some of the repetition that happens just because random is random.
-
         # Calculate a random sampling of lines that are to have text applied
-        # onto them. Return value is a shuffled list of line index integers.
+        # onto them. Return value is a sorted list of line index integers.
         linelen = len(self.lines)
-        affected = random.sample(range(linelen), int(linelen / 3.5))
+        affected = sorted(random.sample(range(linelen), int(linelen / 3.5)))
 
-        # Choose what lines to apply real system data to. Check if a sampling
-        # is possible, since random.sample() will crash if you ask for more
-        # candidates than are available in the list. This makes doge crash if
-        # you have a small selection of affected lines but a lot of system
-        # data.
-        real_targets = self.real_data
-        if len(affected) > len(real_targets):
-            real_targets = random.sample(affected, len(self.real_data))
-
-        for x in affected:
-            line = self.lines[x]
+        for i, target in enumerate(affected, start=1):
+            line = self.lines[target]
             line = re.sub('\n', ' ', line)
 
-            word = None
+            word = self.words.get()
 
-            # Use real data, but apply some jittering to add to the randomness.
-            if x in real_targets and random.choice(range(2)) == 0:
-                word = self.real_data.pop()
-
-            # If not, then add a standalone wow. This will be jittered by the
-            # fact that the above condition must fail.
-            elif x in affected[-2:]:
+            # If first or last line, or a random selection, use standalone wow.
+            if i == 1 or i == len(affected) or random.choice(range(20)) == 0:
                 word = 'wow'
 
             # Generate a new DogeMessage, possibly based on a word.
-            self.lines[x] = DogeMessage(
-                self.tty, line, word=word,
-                extra_words=self.extra_words).generate()
+            self.lines[target] = DogeMessage(self, line, word).generate()
 
     def load_doge(self):
         """
@@ -176,35 +153,32 @@ class Doge(object):
 
         """
 
+        ret = []
         username = os.environ.get('USER')
         if username:
-            self.real_data.append(username)
+            ret.append(username)
 
         editor = os.environ.get('EDITOR')
         if editor:
             editor = editor.split('/')[-1]
-            self.real_data.append(editor)
+            ret.append(editor)
 
         # OS, hostname and... architechture (because lel)
         uname = os.uname()
-        self.real_data.append(uname[0])
-        self.real_data.append(uname[1])
-        self.real_data.append(uname[4])
+        ret.append(uname[0])
+        ret.append(uname[1])
+        ret.append(uname[4])
 
         # Grab actual files from $HOME.
-        files = listdir(os.environ.get('HOME'))
+        files = os.listdir(os.environ.get('HOME'))
         if files:
-            self.real_data.append(random.choice(files))
+            ret.append(random.choice(files))
 
-        # Scan if any of the known processes are running.
-        for proc in self.get_processes():
-            if proc in wow.KNOWN_PROCESSES:
-                self.real_data.append(proc)
-                break
+        # Grab some processes
+        ret += self.get_processes()[:2]
 
-        # Shuffle all the data, lowercase it, and set it.
-        random.shuffle(self.real_data)
-        self.real_data = list(map(str.lower, self.real_data))
+        # Lowercase the data, and set it into the wordlist.
+        self.words.extend(map(str.lower, ret))
 
     def get_stdin_data(self):
         """
@@ -214,7 +188,7 @@ class Doge(object):
 
         if self.tty.in_is_tty:
             # No pipez found
-            return
+            return False
 
         if sys.version_info < (3, 0):
             stdin_lines = (l.decode('utf-8') for l in sys.stdin.xreadlines())
@@ -222,12 +196,18 @@ class Doge(object):
             stdin_lines = (l for l in sys.stdin.readlines())
 
         rx_word = re.compile("\w+", re.UNICODE)
-        self.extra_words.extend([
+
+        # If we have stdin data, we should remove everything else!
+        self.words.clear()
+
+        self.words.extend([
             match.group(0)
             for line in stdin_lines
             for match in rx_word.finditer(line.lower())
         ])
 
+        return True
+
     def get_processes(self):
         """
         Grab a shuffled list of all currently running process names
@@ -246,7 +226,8 @@ class Doge(object):
 
             for comm in output.split('\n'):
                 name = comm.split('/')[-1]
-                if name:
+                # Filter short and weird ones
+                if name and len(name) >= 2 and ':' not in name:
                     procs.add(name)
 
         finally:
@@ -267,27 +248,23 @@ class DogeMessage(object):
 
     """
 
-    def __init__(self, tty, occupied, word=None, extra_words=tuple()):
-        self.tty = tty
+    def __init__(self, doge, occupied, word):
+        self.doge = doge
+        self.tty = doge.tty
         self.occupied = occupied
         self.word = word
-        self.extra_words = extra_words
 
     def generate(self):
         if self.word == 'wow':
             # Standalone wow. Don't apply any prefixes or suffixes.
             msg = self.word
         else:
-            if not self.word:
-                # No word has been set, so grab one randomly from the wordlist.
-                self.word = random.choice(tuple(self.extra_words) + wow.WORDS)
-
             # Add a prefix.
-            msg = u'{0} {1}'.format(random.choice(wow.PREFIXES), self.word)
+            msg = u'{0} {1}'.format(wow.PREFIXES.get(), self.word)
 
             # Seldomly add a suffix as well.
             if random.choice(range(15)) == 0:
-                msg += u' {0}'.format(random.choice(wow.SUFFIXES))
+                msg += u' {0}'.format(wow.SUFFIXES.get())
 
         # Calculate the maximum possible spacer
         interval = self.tty.width - len(msg) - clean_len(self.occupied)
@@ -304,7 +281,7 @@ class DogeMessage(object):
         if self.tty.out_is_tty:
             # Apply pretty ANSI color coding.
             msg = u'\x1b[1m\x1b[38;5;{0}m{1}\x1b[39m\x1b[0m'.format(
-                random.choice(wow.COLORS), msg
+                wow.COLORS.get(), msg
             )
 
         # Line ends are pretty cool guys, add one of those.
@@ -375,7 +352,7 @@ def main():
         print()
 
         lang = os.environ.get('LANG')
-        if not lang and False:
+        if not lang:
             print('wow error: broken $LANG, so fail')
             return 3
 
